.include "jet.h.s"
.include "ai.h.s"
.include "cpctelera.h.s"
.include "globals.s"
.include "sys/physics.h.s"
.include "sys/map.h.s"

ai_storage_size = max_ai * sizeof_ai + 1

ai_next_free::  .dw ai_storage        ;; puntero a la primera entidad que hay libre
ai_storage::   .ds ai_storage_size   ;; reservo memoria del tamaño de las entidades
invalid_end_ai_signal:: .db e_type_invalid  


ai_col_0:: .db 00
ai_col_1:: .db 00
ai_col_20:: .db 00
ai_col_21:: .db 00


man_ai_init::
    ld hl,#ai_storage
    ld (ai_next_free), hl
    ;; Init the complete jet array with "e_type_invalid"
    ld (hl), #e_type_invalid
    ld d,h
    ld e,l
    inc de
    ld bc, #ai_storage_size - 1
    ldir ;; coge lo que hay en hl y lo copia en de tantos bytes como diga bc

    ret

man_ai_create:: ;;ld ix,#ai_storage BEFORE CALL
    ld e_type(ix), #e_type_default
    ld e_pos(ix), #j_pos_3
    ld e_x(ix), #35
    ld e_y(ix), #15
    ld e_vx(ix), #0
    ld e_vy(ix), #0
    ld e_w(ix), #4
    ld e_h(ix), #8
    ld e_prev_x(ix), #0x00
    ld e_prev_y(ix), #0xC0
    ld e_prev_pos(ix), #3
    ld ai_section(ix), #2
    ld ai_cont_section(ix), #0
    ld ai_turn_mode(ix),#0
    ld a,e_y(ix)
    sub #0x08
    ld ai_collision_y_0(ix), a
    add a, #0x18
    ld ai_collision_y_60(ix),a ;se guarda 2 mas abajo por falta de memoria
    ld a,e_x(ix)
    sub #0x04
    ld ai_collision_x(ix), a
    ret

man_ai_destroy::
    ld d, h
    ld e,l
    ld a,#0x00
    ld bc,#sizeof_ai
    call cpct_memset_asm
    ret

man_ai_turn::

    ld a,ai_turn_mode(ix)
    cp #0
    jr z,ai_comp_turn
        ld a,ai_turn_cont(ix)
        cp #0
        jr nz, ai_turn_cont_continue
            call cpct_getRandom_xsp40_u8_asm
            and #0b00000001
            ld ai_turn_time_cont(ix),a
        ai_turn_cont_continue:
            ld a,ai_turn_time_cont(ix)
            cp ai_turn_cont(ix)
            jr z, ai_turn_action
            inc ai_turn_cont(ix)
            ret

        ai_turn_action:
        ld a,ai_turn_mode(ix)
        cp #1
        jr z,ai_call_turn_left
            call sys_physics_ai_turn_right
            ret
        ai_call_turn_left:
            call sys_physics_ai_turn_left
            ret

    ai_comp_turn:

    ld ai_turn_cont(ix),#0

    ld a,ai_collision_x(ix)
    srl a
    srl a
    ld c,a
    ld b,#0
    ld a,ai_section(ix)
    cp #3
    jr z,load_collision_y_60
    ld a,ai_collision_y_0(ix)
    jr end_ld
    load_collision_y_60:
    ld a,ai_collision_y_60(ix)
    end_ld:
    and #0b11111000
    ld e,a
    srl e
    ld d,b
    ld l,a
    ld h,d
    add hl,hl
    add hl,de
    add hl,bc
    ex de,hl
    ld iy, #tilemap_ptr
    add iy,de

    ld a,ai_section(ix)
    cp #1
    jr z, turn_1
    cp #2
    jr z, turn_2
    cp #3
    jr z, turn_3
    cp #4
    jr z,turn_4
    ret


    turn_1:
    ld a,2-80(iy)
    or 2-40(iy)
    or 2-20(iy)
    or 2(iy)
    cp #0
    ret z
    cp #0x0A
    ret z
        ld a, 25-80(iy)
        or 24-80(iy)
        or 23-80(iy)
        cp #0
        jp nz, ai_turn_overclock
        jp ai_turn_clock

    turn_2:
    ld a,22+4(iy)
    or 23+4(iy)
    cp #0
    ret z
    cp #0x0A
    ret z
        ld a,-20+4(iy)
        or 4(iy)
        or 20+4(iy)
        cp #0
        jp nz, ai_turn_clock
        jp ai_turn_overclock

    turn_3: 
    ld a,102(iy)
    or 42(iy)
    
    or 82(iy)
    ;
    cp #0
    ret z
    cp #0x0A
    ret z
        ld a,45+40(iy)
        or 44+40(iy)
        or 43+40(iy)
        or 42+40(iy)
        cp #0
        jp nz, ai_turn_clock
        jp ai_turn_overclock

    turn_4:
    ld a,20-5(iy)
    or 21-5(iy)
    cp #0
    ret z
    cp #0x0A
    ret z
        ld a,-2(iy)
        or a,-2+20(iy)
        cp #0
        jp nz,ai_turn_overclock
        jp ai_turn_clock
    
    ai_turn_clock:
        ld ai_turn_mode(ix),#2
        call sys_physics_ai_turn_right
        ret

    ai_turn_overclock:
        ld ai_turn_mode(ix),#1
        call sys_physics_ai_turn_left
        ret
    ret
    

 man_ai_map01::
    ld ix,#ai_storage
     ld e_x(ix), #35
    ld e_y(ix), #15
    ld e_vx(ix), #0
    ld e_vy(ix), #0
    ld e_w(ix), #4
    ld e_h(ix), #8
    ld e_prev_x(ix), #0x00
    ld e_prev_y(ix), #0xC0
    ld e_pos(ix),#3
    ld e_prev_pos(ix), #3
    ld ai_section(ix), #2
    ld ai_cont_section(ix), #0
    ld ai_turn_mode(ix),#0
    ld a,e_y(ix)
    sub #0x08
    ld ai_collision_y_0(ix), a
    add a, #0x18
    ld ai_collision_y_60(ix),a ;se guarda 2 mas abajo por falta de memoria
    ld a,e_x(ix)
    sub #0x04
    ld ai_collision_x(ix), a
    ret 
man_ai_map02::
    ld ix,#ai_storage
    ld e_x(ix), #60
    ld e_y(ix), #137
    ld e_vx(ix), #0
    ld e_vy(ix), #0
    ld e_prev_pos(ix), #9
    ld e_pos(ix), #9
    ld ai_section(ix), #4
    ld ai_cont_section(ix), #0
    ld ai_turn_mode(ix),#0
    ld a,e_y(ix)
    sub #0x08
    ld ai_collision_y_0(ix), a
    add a, #0x18
    ld ai_collision_y_60(ix),a ;se guarda 2 mas abajo por falta de memoria
    ld a,e_x(ix)
    sub #0x04
    ld ai_collision_x(ix), a
    ret

man_ai_map03::
    ld ix,#ai_storage
    ld e_x(ix), #20
    ld e_y(ix), #15
    ld e_prev_pos(ix), #3
    ld e_vx(ix), #0
    ld e_vy(ix), #0
    ld e_pos(ix), #3
    ld ai_section(ix), #2
    ld ai_cont_section(ix), #0
    ld ai_turn_mode(ix),#0
    ld a,e_y(ix)
    sub #0x08
    ld ai_collision_y_0(ix), a
    add a, #0x18
    ld ai_collision_y_60(ix),a ;se guarda 2 mas abajo por falta de memoria
    ld a,e_x(ix)
    sub #0x04
    ld ai_collision_x(ix), a
    ret